@@ -131,6 +131,17 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
         self.subarrays.len()
     }
 
+    pub fn lengths<'a>(
+        &self,
+        py: Python<'a>,
+    ) -> &'a numpy::PyArray<usize, numpy::ndarray::Dim<[usize; 1]>> {
+        self.subarrays
+            .iter()
+            .map(|r| (r.end - r.start) / self.features)
+            .collect::<Vec<_>>()
+            .to_pyarray(py)
+    }
+
     pub fn size1(&self, i: usize) -> PyResult<usize> {
         if i >= self.subarrays.len() {
             Err(pyo3::exceptions::PyIndexError::new_err(format!(
@@ -243,4 +254,118 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
             )))
         }
     }
+
+    pub fn add_scalar(&self, scalar: T) -> RaggedBuffer<T> {
+        RaggedBuffer {
+            data: self.data.iter().map(|x| *x + scalar).collect(),
+            subarrays: self.subarrays.clone(),
+            features: self.features,
+        }
+    }
+
+    pub fn indices(&self, dim: usize) -> PyResult<RaggedBuffer<i64>> {
+        match dim {
+            0 => {
+                let mut indices = Vec::with_capacity(self.data.len() / self.features);
+                for (index, subarray) in self.subarrays.iter().enumerate() {
+                    for _ in 0..(subarray.end - subarray.start) / self.features {
+                        indices.push(index as i64);
+                    }
+                }
+                Ok(RaggedBuffer {
+                    subarrays: self
+                        .subarrays
+                        .iter()
+                        .map(|r| r.start / self.features..r.end / self.features)
+                        .collect(),
+                    data: indices,
+                    features: 1,
+                })
+            }
+            1 => Err(pyo3::exceptions::PyValueError::new_err(
+                "Sequence indices not yet implemented",
+            )),
+            _ => Err(pyo3::exceptions::PyValueError::new_err(format!(
+                "Invalid dimension {}",
+                dim
+            ))),
+        }
+    }
+
+    pub fn flat_indices(&self) -> PyResult<RaggedBuffer<i64>> {
+        Ok(RaggedBuffer {
+            subarrays: self
+                .subarrays
+                .iter()
+                .map(|r| r.start / self.features..r.end / self.features)
+                .collect(),
+            data: (0..self.data.len() / self.features)
+                .map(|i| i as i64)
+                .collect(),
+            features: 1,
+        })
+    }
+
+    pub fn cat(
+        lhs: &RaggedBuffer<T>,
+        rhs: &RaggedBuffer<T>,
+        dim: usize,
+    ) -> PyResult<RaggedBuffer<T>> {
+        match dim {
+            0 => {
+                let mut data = Vec::with_capacity(lhs.data.len() + rhs.data.len());
+                data.extend_from_slice(&lhs.data);
+                data.extend_from_slice(&rhs.data);
+                Ok(RaggedBuffer {
+                    data,
+                    subarrays: lhs
+                        .subarrays
+                        .iter()
+                        .cloned()
+                        .chain(
+                            rhs.subarrays
+                                .iter()
+                                .map(|r| r.start + lhs.data.len()..r.end + lhs.data.len()),
+                        )
+                        .collect(),
+                    features: lhs.features,
+                })
+            }
+            1 => {
+                if lhs.subarrays.len() != rhs.subarrays.len() {
+                    return Err(pyo3::exceptions::PyValueError::new_err(format!(
+                        "Size of dimension 1 must match for all buffers, got {} != {}",
+                        lhs.subarrays.len(),
+                        rhs.subarrays.len()
+                    )));
+                }
+                if lhs.features != rhs.features {
+                    return Err(pyo3::exceptions::PyValueError::new_err(format!(
+                        "Number of features must match for all buffers, got {} != {}",
+                        lhs.features, rhs.features
+                    )));
+                }
+                let mut data = Vec::with_capacity(lhs.data.len() + rhs.data.len());
+                let mut subarrays = Vec::with_capacity(lhs.subarrays.len());
+                for (lhs_subarray, rhs_subarray) in lhs.subarrays.iter().zip(rhs.subarrays.iter()) {
+                    let start = data.len();
+                    data.extend_from_slice(&lhs.data[lhs_subarray.clone()]);
+                    data.extend_from_slice(&rhs.data[rhs_subarray.clone()]);
+                    subarrays.push(start..data.len());
+                }
+                Ok(RaggedBuffer {
+                    data,
+                    subarrays,
+                    features: lhs.features,
+                })
+            }
+            2 => Err(pyo3::exceptions::PyValueError::new_err(
+                "Concatenation along dimension 2 not yet implemented",
+            )),
+            _ => Err(pyo3::exceptions::PyValueError::new_err(format!(
+                "Invalid dimension {}, RaggedBuffer only has 3 dimensions",
+                dim
+            ))),
+        }
+    }
 }
